@@ -65,7 +65,6 @@ const USER_DEFINED_METADATA_HEADER_PREFIX: &str = "x-amz-meta-";
 
 /// A specialized `Error` for object store-related errors
 #[derive(Debug, Snafu)]
-#[allow(missing_docs)]
 pub(crate) enum Error {
     #[snafu(display("Error performing DeleteObjects request: {}", source))]
     DeleteObjectsRequest { source: crate::client::retry::Error },
@@ -166,7 +165,7 @@ impl From<DeleteError> for Error {
 }
 
 #[derive(Debug)]
-pub struct S3Config {
+pub(crate) struct S3Config {
     pub region: String,
     pub endpoint: Option<String>,
     pub bucket: String,
@@ -181,7 +180,7 @@ pub struct S3Config {
     pub checksum: Option<Checksum>,
     pub copy_if_not_exists: Option<S3CopyIfNotExists>,
     pub conditional_put: Option<S3ConditionalPut>,
-    pub encryption_headers: S3EncryptionHeaders,
+    pub(super) encryption_headers: S3EncryptionHeaders,
 }
 
 impl S3Config {
@@ -267,15 +266,16 @@ pub(crate) struct Request<'a> {
     payload: Option<PutPayload>,
     use_session_creds: bool,
     idempotent: bool,
+    retry_error_body: bool,
 }
 
 impl<'a> Request<'a> {
-    pub fn query<T: Serialize + ?Sized + Sync>(self, query: &T) -> Self {
+    pub(crate) fn query<T: Serialize + ?Sized + Sync>(self, query: &T) -> Self {
         let builder = self.builder.query(query);
         Self { builder, ..self }
     }
 
-    pub fn header<K>(self, k: K, v: &str) -> Self
+    pub(crate) fn header<K>(self, k: K, v: &str) -> Self
     where
         HeaderName: TryFrom<K>,
         <HeaderName as TryFrom<K>>::Error: Into<http::Error>,
@@ -284,29 +284,36 @@ impl<'a> Request<'a> {
         Self { builder, ..self }
     }
 
-    pub fn headers(self, headers: HeaderMap) -> Self {
+    pub(crate) fn headers(self, headers: HeaderMap) -> Self {
         let builder = self.builder.headers(headers);
         Self { builder, ..self }
     }
 
-    pub fn idempotent(self, idempotent: bool) -> Self {
+    pub(crate) fn idempotent(self, idempotent: bool) -> Self {
         Self { idempotent, ..self }
     }
 
-    pub fn with_encryption_headers(self) -> Self {
+    pub(crate) fn retry_error_body(self, retry_error_body: bool) -> Self {
+        Self {
+            retry_error_body,
+            ..self
+        }
+    }
+
+    pub(crate) fn with_encryption_headers(self) -> Self {
         let headers = self.config.encryption_headers.clone().into();
         let builder = self.builder.headers(headers);
         Self { builder, ..self }
     }
 
-    pub fn with_session_creds(self, use_session_creds: bool) -> Self {
+    pub(crate) fn with_session_creds(self, use_session_creds: bool) -> Self {
         Self {
             use_session_creds,
             ..self
         }
     }
 
-    pub fn with_tags(mut self, tags: TagSet) -> Self {
+    pub(crate) fn with_tags(mut self, tags: TagSet) -> Self {
         let tags = tags.encoded();
         if !tags.is_empty() && !self.config.disable_tagging {
             self.builder = self.builder.header(&TAGS_HEADER, tags);
@@ -314,7 +321,7 @@ impl<'a> Request<'a> {
         self
     }
 
-    pub fn with_attributes(self, attributes: Attributes) -> Self {
+    pub(crate) fn with_attributes(self, attributes: Attributes) -> Self {
         let mut has_content_type = false;
         let mut builder = self.builder;
         for (k, v) in &attributes {
@@ -342,7 +349,7 @@ impl<'a> Request<'a> {
         Self { builder, ..self }
     }
 
-    pub fn with_payload(mut self, payload: PutPayload) -> Self {
+    pub(crate) fn with_payload(mut self, payload: PutPayload) -> Self {
         if !self.config.skip_signature || self.config.checksum.is_some() {
             let mut sha256 = Context::new(&digest::SHA256);
             payload.iter().for_each(|x| sha256.update(x));
@@ -363,7 +370,7 @@ impl<'a> Request<'a> {
         self
     }
 
-    pub async fn send(self) -> Result<Response, RequestError> {
+    pub(crate) async fn send(self) -> Result<Response, RequestError> {
         let credential = match self.use_session_creds {
             true => self.config.get_session_credential().await?,
             false => SessionCredential {
@@ -380,13 +387,14 @@ impl<'a> Request<'a> {
             .with_aws_sigv4(credential.authorizer(), sha)
             .retryable(&self.config.retry_config)
             .idempotent(self.idempotent)
+            .retry_error_body(self.retry_error_body)
             .payload(self.payload)
             .send()
             .await
             .context(RetrySnafu { path })
     }
 
-    pub async fn do_put(self) -> Result<PutResult> {
+    pub(crate) async fn do_put(self) -> Result<PutResult> {
         let response = self.send().await?;
         Ok(get_put_result(response.headers(), VERSION_HEADER).context(MetadataSnafu)?)
     }
@@ -399,12 +407,12 @@ pub(crate) struct S3Client {
 }
 
 impl S3Client {
-    pub fn new(config: S3Config) -> Result<Self> {
+    pub(crate) fn new(config: S3Config) -> Result<Self> {
         let client = config.client_options.client()?;
         Ok(Self { config, client })
     }
 
-    pub fn request<'a>(&'a self, method: Method, path: &'a Path) -> Request<'a> {
+    pub(crate) fn request<'a>(&'a self, method: Method, path: &'a Path) -> Request<'a> {
         let url = self.config.path_url(path);
         Request {
             path,
@@ -414,6 +422,7 @@ impl S3Client {
             config: &self.config,
             use_session_creds: true,
             idempotent: false,
+            retry_error_body: false,
         }
     }
 
@@ -424,7 +433,7 @@ impl S3Client {
     /// there was an error for a certain path, the error will be returned in the
     /// vector. If there was an issue with making the overall request, an error
     /// will be returned at the top level.
-    pub async fn bulk_delete_request(&self, paths: Vec<Path>) -> Result<Vec<Result<Path>>> {
+    pub(crate) async fn bulk_delete_request(&self, paths: Vec<Path>) -> Result<Vec<Result<Path>>> {
         if paths.is_empty() {
             return Ok(Vec::new());
         }
@@ -520,16 +529,54 @@ impl S3Client {
     }
 
     /// Make an S3 Copy request <https://docs.aws.amazon.com/AmazonS3/latest/API/API_CopyObject.html>
-    pub fn copy_request<'a>(&'a self, from: &Path, to: &'a Path) -> Request<'a> {
+    pub(crate) fn copy_request<'a>(&'a self, from: &Path, to: &'a Path) -> Request<'a> {
         let source = format!("{}/{}", self.config.bucket, encode_path(from));
+
+        let mut copy_source_encryption_headers = HeaderMap::new();
+        if let Some(customer_algorithm) = self
+            .config
+            .encryption_headers
+            .0
+            .get("x-amz-server-side-encryption-customer-algorithm")
+        {
+            copy_source_encryption_headers.insert(
+                "x-amz-copy-source-server-side-encryption-customer-algorithm",
+                customer_algorithm.clone(),
+            );
+        }
+        if let Some(customer_key) = self
+            .config
+            .encryption_headers
+            .0
+            .get("x-amz-server-side-encryption-customer-key")
+        {
+            copy_source_encryption_headers.insert(
+                "x-amz-copy-source-server-side-encryption-customer-key",
+                customer_key.clone(),
+            );
+        }
+        if let Some(customer_key_md5) = self
+            .config
+            .encryption_headers
+            .0
+            .get("x-amz-server-side-encryption-customer-key-MD5")
+        {
+            copy_source_encryption_headers.insert(
+                "x-amz-copy-source-server-side-encryption-customer-key-MD5",
+                customer_key_md5.clone(),
+            );
+        }
+
         self.request(Method::PUT, to)
             .idempotent(true)
+            .retry_error_body(true)
             .header(&COPY_SOURCE_HEADER, &source)
             .headers(self.config.encryption_headers.clone().into())
+            .headers(copy_source_encryption_headers)
             .with_session_creds(false)
     }
 
-    pub async fn create_multipart(
+    pub(crate) async fn create_multipart(
         &self,
         location: &Path,
         opts: PutMultipartOpts,
@@ -553,7 +600,7 @@ impl S3Client {
         Ok(response.upload_id)
     }
 
-    pub async fn put_part(
+    pub(crate) async fn put_part(
         &self,
         path: &Path,
         upload_id: &MultipartId,
@@ -562,19 +609,27 @@ impl S3Client {
     ) -> Result<PartId> {
         let part = (part_idx + 1).to_string();
 
-        let response = self
+        let mut request = self
             .request(Method::PUT, path)
             .with_payload(data)
             .query(&[("partNumber", &part), ("uploadId", upload_id)])
-            .idempotent(true)
-            .send()
-            .await?;
+            .idempotent(true);
+        if self
+            .config
+            .encryption_headers
+            .0
+            .contains_key("x-amz-server-side-encryption-customer-algorithm")
+        {
+            // If SSE-C is used, we must include the encryption headers in every upload request.
+            request = request.with_encryption_headers();
+        }
+        let response = request.send().await?;
 
         let content_id = get_etag(response.headers()).context(MetadataSnafu)?;
         Ok(PartId { content_id })
     }
 
-    pub async fn complete_multipart(
+    pub(crate) async fn complete_multipart(
         &self,
         location: &Path,
         upload_id: &str,
@@ -604,6 +659,7 @@ impl S3Client {
             .with_aws_sigv4(credential.authorizer(), None)
             .retryable(&self.config.retry_config)
             .idempotent(true)
+            .retry_error_body(true)
             .send()
             .await
             .context(CompleteMultipartRequestSnafu)?;
@@ -625,7 +681,7 @@ impl S3Client {
     }
 
     #[cfg(test)]
-    pub async fn get_object_tagging(&self, path: &Path) -> Result<Response> {
+    pub(crate) async fn get_object_tagging(&self, path: &Path) -> Result<Response> {
         let credential = self.config.get_session_credential().await?;
         let url = format!("{}?tagging", self.config.path_url(path));
         let response = self
@@ -660,6 +716,14 @@ impl GetClient for S3Client {
         };
 
         let mut builder = self.client.request(method, url);
+        if self
+            .config
+            .encryption_headers
+            .0
+            .contains_key("x-amz-server-side-encryption-customer-algorithm")
+        {
+            builder = builder.headers(self.config.encryption_headers.clone().into());
+        }
 
         if let Some(v) = &options.version {
             builder = builder.query(&[("versionId", v)])
