@@ -124,7 +124,7 @@ impl From<Error> for crate::Error {
 }
 
 #[derive(Debug)]
-pub struct GoogleCloudStorageConfig {
+pub(crate) struct GoogleCloudStorageConfig {
     pub base_url: String,
 
     pub credentials: GcpCredentialProvider,
@@ -139,7 +139,7 @@ pub struct GoogleCloudStorageConfig {
 }
 
 impl GoogleCloudStorageConfig {
-    pub fn new(
+    pub(crate) fn new(
         base_url: String,
         credentials: GcpCredentialProvider,
         signing_credentials: GcpSigningCredentialProvider,
@@ -157,13 +157,13 @@ impl GoogleCloudStorageConfig {
         }
     }
 
-    pub fn path_url(&self, path: &Path) -> String {
+    pub(crate) fn path_url(&self, path: &Path) -> String {
         format!("{}/{}/{}", self.base_url, self.bucket_name, path)
     }
 }
 
 /// A builder for a put request allowing customisation of the headers and query string
-pub struct Request<'a> {
+pub(crate) struct Request<'a> {
     path: &'a Path,
     config: &'a GoogleCloudStorageConfig,
     payload: Option<PutPayload>,
@@ -261,7 +261,7 @@ struct SignBlobResponse {
 }
 
 #[derive(Debug)]
-pub struct GoogleCloudStorageClient {
+pub(crate) struct GoogleCloudStorageClient {
     config: GoogleCloudStorageConfig,
 
     client: Client,
@@ -273,7 +273,7 @@ pub struct GoogleCloudStorageClient {
 }
 
 impl GoogleCloudStorageClient {
-    pub fn new(config: GoogleCloudStorageConfig) -> Result<Self> {
+    pub(crate) fn new(config: GoogleCloudStorageConfig) -> Result<Self> {
         let client = config.client_options.client()?;
         let bucket_name_encoded =
             percent_encode(config.bucket_name.as_bytes(), NON_ALPHANUMERIC).to_string();
@@ -286,7 +286,7 @@ impl GoogleCloudStorageClient {
         })
     }
 
-    pub fn config(&self) -> &GoogleCloudStorageConfig {
+    pub(crate) fn config(&self) -> &GoogleCloudStorageConfig {
         &self.config
     }
 
@@ -309,7 +309,11 @@ impl GoogleCloudStorageClient {
     ///  "payload": "REQUEST_INFORMATION"
     /// }
     /// ```
-    pub async fn sign_blob(&self, string_to_sign: &str, client_email: &str) -> Result<String> {
+    pub(crate) async fn sign_blob(
+        &self,
+        string_to_sign: &str,
+        client_email: &str,
+    ) -> Result<String> {
         let credential = self.get_credential().await?;
         let body = SignBlobBody {
             payload: BASE64_STANDARD.encode(string_to_sign),
@@ -344,7 +348,7 @@ impl GoogleCloudStorageClient {
         Ok(hex_encode(&signed_blob))
     }
 
-    pub fn object_url(&self, path: &Path) -> String {
+    pub(crate) fn object_url(&self, path: &Path) -> String {
         let encoded = utf8_percent_encode(path.as_ref(), NON_ALPHANUMERIC);
         format!(
             "{}/{}/{}",
@@ -355,7 +359,7 @@ impl GoogleCloudStorageClient {
     /// Perform a put request <https://cloud.google.com/storage/docs/xml-api/put-object-upload>
     ///
     /// Returns the new ETag
-    pub fn request<'a>(&'a self, method: Method, path: &'a Path) -> Request<'a> {
+    pub(crate) fn request<'a>(&'a self, method: Method, path: &'a Path) -> Request<'a> {
         let builder = self.client.request(method, self.object_url(path));
 
         Request {
@@ -367,7 +371,7 @@ impl GoogleCloudStorageClient {
         }
     }
 
-    pub async fn put(
+    pub(crate) async fn put(
         &self,
         path: &Path,
         payload: PutPayload,
@@ -398,7 +402,7 @@ impl GoogleCloudStorageClient {
     /// Perform a put part request <https://cloud.google.com/storage/docs/xml-api/put-object-multipart>
     ///
     /// Returns the new [`PartId`]
-    pub async fn put_part(
+    pub(crate) async fn put_part(
         &self,
         path: &Path,
         upload_id: &MultipartId,
@@ -423,7 +427,7 @@ impl GoogleCloudStorageClient {
     }
 
     /// Initiate a multipart upload <https://cloud.google.com/storage/docs/xml-api/post-object-multipart>
-    pub async fn multipart_initiate(
+    pub(crate) async fn multipart_initiate(
         &self,
         path: &Path,
         opts: PutMultipartOpts,
@@ -444,7 +448,11 @@ impl GoogleCloudStorageClient {
     }
 
     /// Cleanup unused parts <https://cloud.google.com/storage/docs/xml-api/delete-multipart>
-    pub async fn multipart_cleanup(&self, path: &Path, multipart_id: &MultipartId) -> Result<()> {
+    pub(crate) async fn multipart_cleanup(
+        &self,
+        path: &Path,
+        multipart_id: &MultipartId,
+    ) -> Result<()> {
         let credential = self.get_credential().await?;
         let url = self.object_url(path);
 
@@ -463,7 +471,7 @@ impl GoogleCloudStorageClient {
         Ok(())
     }
 
-    pub async fn multipart_complete(
+    pub(crate) async fn multipart_complete(
         &self,
         path: &Path,
         multipart_id: &MultipartId,
@@ -522,13 +530,18 @@ impl GoogleCloudStorageClient {
     }
 
     /// Perform a delete request <https://cloud.google.com/storage/docs/xml-api/delete-object>
-    pub async fn delete_request(&self, path: &Path) -> Result<()> {
+    pub(crate) async fn delete_request(&self, path: &Path) -> Result<()> {
         self.request(Method::DELETE, path).send().await?;
         Ok(())
     }
 
     /// Perform a copy request <https://cloud.google.com/storage/docs/xml-api/put-object-copy>
-    pub async fn copy_request(&self, from: &Path, to: &Path, if_not_exists: bool) -> Result<()> {
+    pub(crate) async fn copy_request(
+        &self,
+        from: &Path,
+        to: &Path,
+        if_not_exists: bool,
+    ) -> Result<()> {
         let credential = self.get_credential().await?;
         let url = self.object_url(to);
 
